import argparse
import tablib
import sys

from irco import graphs


def write(fh, graph, format=None):
    from networkx.readwrite import gexf
    gexf.write_gexf(graph, fh)


def main():
    format_choices = []
    graph_choices = graphs.get_graphs()
    argparser = argparse.ArgumentParser('irco-graph')
    argparser.add_argument('-f', '--format', choices=format_choices)
    argparser.add_argument('graph_type', choices=graph_choices)
    argparser.add_argument('source')
    argparser.add_argument('output', default='-', nargs='?')

    args = argparser.parse_args()

    with open(args.source, 'r') as fh:
        dataset = tablib.import_set(fh)

    graph_factory = graphs.get_graph(args.graph_type)
    graph = graph_factory.create(dataset)

    if args.output == '-':
        write(sys.stdout, graph)
    else:
        with open(args.output, 'wb') as fh:
            write(fh, graph)



import argparse
import os
import glob
import sys
from tablib import formats

from irco import parser, tabular


def get_file_list(sources):
    for source in sources:
        if os.path.isdir(source):
            for path in glob.glob(os.path.join(source, '*.txt')):
                yield path
        elif os.path.isfile(source):
            yield source


def get_format(args, choices):
    format = args.format
    if not format:
        # Guess it from file extension
        try:
            format = args.output.rsplit('.', 1)[1]
        except IndexError:
            if args.output != '-':
                print 'Could not guess output format from filename.'
            else:
                print 'Cannot guess output format when writing to stdout.'
            print 'Please use the -f/--format option to specify it.'
            sys.exit(1)
        else:
            if format not in choices:
                print ('Could not guess output format from filename ({}).'
                       .format(format))
                print 'Please use the -f/--format option to specify it.'
                sys.exit(1)
    return format


def get_dataset(source):
    table = tabular.Table(notset=None)
    for path in get_file_list(source):
        with open(path) as fh:
            for record in parser.parse(fh):
                table.add(record)
    return table.dataset()


def write(fh, dataset, format):
    fh.write(getattr(dataset, format))


def main():
    format_choices = [fmt.title for fmt in formats.available]
    argparser = argparse.ArgumentParser('irco-convert')
    argparser.add_argument('-f', '--format', choices=format_choices)
    argparser.add_argument('source', nargs='+')
    argparser.add_argument('output')

    args = argparser.parse_args()

    format = get_format(args, format_choices)
    dataset = get_dataset(args.source)

    if args.output == '-':
        write(sys.stdout, dataset, format)
    else:
        with open(args.output, 'wb') as fh:
            write(fh, dataset, format)



import collections
from itertools import izip, combinations

import networkx as nx


def get_country(text):
    return text.rsplit(', ')[-1]


def get_countries(affiliations):
    return set(get_country(a) for a in affiliations)


def create(dataset):
    g = nx.Graph()

    coauthors = dataset['authors']
    affiliation_sets = dataset['author_affiliation']

    countries = set()

    papers_count = collections.Counter()
    collaborations_count = collections.Counter()

    for authors, affiliations in izip(coauthors, affiliation_sets):
        author_names = []
        for author, affiliation_id in authors:
            affiliation = affiliations[str(affiliation_id)]
            country = get_country(affiliation)
            countries.add(country)
            papers_count[author] += 1
            g.add_node(author, affiliation_country=country)
            author_names.append(author)

        collaborations = list(combinations(author_names, 2))
        collaborations_count.update(collaborations)
        g.add_edges_from(collaborations)

    countries = {c: i for i, c in enumerate(countries)}

    # Set papers count
    for author, count in papers_count.iteritems():
        n = g.node[author]
        n['papers'] = count
        n['affiliation_country_id'] = countries[n['affiliation_country']]

    # Set edge weight
    for (c1, c2), count in collaborations_count.iteritems():
        g[c1][c2]['weight'] = count

    return g



import pipes

from fabric.api import settings, task, local, hide
from fabric.contrib.console import confirm


def is_working_tree_clean():
    with settings(hide('everything'), warn_only=True):
        local('git update-index -q --ignore-submodules --refresh')
        unstaged = local('git diff-files --quiet --ignore-submodules --',
                         capture=True)
        uncommitted = local('git diff-index --cached --quiet HEAD '
                            '--ignore-submodules --', capture=True)
    return unstaged.succeeded and uncommitted.succeeded


@task
def lint():
    """
    Checks the source code using flake8.
    """
    local('flake8 --statistics --exit-zero --max-complexity=10 '
          '--exclude=\'*/migrations/*,build,dist\' .')


@task
def authors():
    """
    Updates the AUTHORS file with a list of committers from GIT.
    """
    local('git shortlog -s -e -n | cut -f 2- > AUTHORS')


@task
def release():
    """
    Create a new release and upload it to PyPI.
    """

    if not is_working_tree_clean():
        print 'Your working tree is not clean. Refusing to create a release.'
        #return

    print 'Rebuilding the AUTHORS file to check for modifications...'
    authors()

    if not is_working_tree_clean():
        print (
            'Your working tree is not clean after the AUTHORS file was '
            'rebuilt.'
        )
        print 'Please commit the changes before continuing.'
        #return

    # Get version
    version = 'v{}'.format(local('python setup.py --version', capture=True))
    name = local('python setup.py --name', capture=True)

    # Tag
    tag_message = '{} release version {}.'.format(name, version)

    print '----------------------'
    print 'Proceeding will tag the release, push the repository upstream,'
    print 'and release a new version on PyPI.'
    print
    print 'Version: {}'.format(version)
    print 'Tag message: {}'.format(tag_message)
    print
    if not confirm('Continue?', default=True):
        print 'Aborting.'
        return

    #local('git tag -a {} -m {}'.format(pipes.quote(version),
    #                                   pipes.quote(tag_message)))

    # Push
    local('git push origin master')

    # Package and upload to pypi
    local('python setup.py sdist upload')



import re


class NamePart(object):
    def __init__(self, name):
        self.name = name

    def is_abbreviated(self):
        return self.name.endswith('.')

    def __unicode__(self):
        return self.name

    def __repr__(self):
        return 'NamePart({!r})'.format(self.name)

    def __ne__(self, other):
        return not (self == other)

    def __hash__(self):
        return hash(self.name)

    def __eq__(self, other):
        if not isinstance(other, NamePart):
            return NotImplemented

        n1, n2 = self.name, other.name

        if n1 == n2:
            return True

        if n1.endswith('.') and n2.startswith(n1[:-1]):
            return True

        if n2.endswith('.') and n1.startswith(n2[:-1]):
            return True

        return False


class Author(str):
    def __init__(self, name):
        match = re.match('^([^\(]+)(?:\(([^@]+@[^ ]+)\))?$', name)
        self.name, self.email = match.groups()
        self.chunks = self.split_name(self.name)

    def split_name(self, name):
        chunks = []
        for c in name.split(', '):
            chunks += self.split_chunk(c)
        chunks = sorted(chunks, key=lambda p: p.name)
        return tuple(chunks)

    def split_chunk(self, chunk):
        parts = [c.strip() for c in re.split(r'([. ]{1,2})', chunk)]

        if parts[-1] == '':
            del parts[-1]
        for i in range(len(parts) - 1, 0, -2):
            parts[i-1] += parts[i]
            del parts[i]

        parts = [NamePart(p) for p in parts]
        parts = sorted(parts, key=lambda p: p.name)
        return tuple(parts)

    def __unicode__(self):
        return self.name

    def __repr__(self):
        return 'Author({})'.format(', '.join([repr(c) for c in self.chunks]))

    def __ne__(self, other):
        return not (self == other)

    def __eq__(self, other):
        if not isinstance(other, Author):
            return False

        if len(self.chunks) != len(other.chunks):
            return False

        self_abbr = set([c for c in self.chunks if c.is_abbreviated()])
        self_not_abbr = set(self.chunks) - self_abbr

        other = set(other.chunks)

        return (self._find_and_remove(self_not_abbr, other) and
                self._find_and_remove(self_abbr, other))

    def __hash__(self):
        initials = ''.join(n.name[0] for n in self.chunks).lower()
        return hash(initials)

    def _find_and_remove(self, set1, set2):
        while set1:
            n1 = set1.pop()
            if n1 in set2:
                set2.remove(n1)
                continue
            else:
                for n2 in set2:
                    if n1 == n2:
                        set2.remove(n2)
                        break
                else:
                    return False
                continue
        return True



import re
from irco.authors import Author


class IgnoreField(Exception):
    pass


class Parser(object):

    KEY_MAPPING = {}

    def __init__(self, value_parser, record_processor):
        self.value_parser = value_parser
        self.record_processor = record_processor

    def get_key(self, k):
        k = k.lower()
        k = re.sub(r'[\s-]+', '_', k)
        return self.KEY_MAPPING.get(k, k)

    def get_value(self, k, v):
        func = getattr(self.value_parser, k, lambda x: x)
        return func(v)

    def parse_line(self, line):
        if line.startswith('Compilation and indexing terms'):
            raise IgnoreField()
        k, v = line.split(':', 1)
        k = self.get_key(k)
        v = self.get_value(k, v)
        return k, v

    def parse_record(self, record_id, record_lines):
        record = {
            'id': record_id,
        }

        for line in record_lines:
            try:
                k, v = self.parse_line(line)
            except ValueError:
                print 'Error in line: {!r}'.format(line)
                continue
            except IgnoreField:
                continue
            record[k] = v

        return record

    def postprocess_record(self, record):
        try:
            return self.record_processor.process(record)
        except Exception as e:
            import pprint
            print
            print '---'
            print 'Error while postprocessing record'
            print
            print e
            print
            pprint.pprint(record)
            print
            raise

    def get_records(self, fh):
        record = None
        record_id = None

        for line in fh:
            match = re.match(r'<RECORD (\d+)>', line)
            if match is not None:
                next_record_id = int(match.group(1))
                if next_record_id > 1:
                    record = self.parse_record(record_id, record)
                    yield self.postprocess_record(record)
                record = []
                record_id = next_record_id
            else:
                line = line.strip()
                if line:
                    record.append(line)

        if record:
            record = self.parse_record(record_id, record)
            yield self.postprocess_record(record)

    def parse(self, fh):
        return self.get_records(fh)


class AuthorList(list):
    def __str__(self):
        return '; '.join(['{0.name} ({1})'.format(*a) for a in self])


class Affiliations(dict):
    def __str__(self):
        return '; '.join(['({}) {}'.format(k, v) for k, v in self.iteritems()])


class ValueParser(object):
    def authors(self, v):
        v = v.split('; ')
        authors = (re.search('^(.*) \((\d+)\)$', a).groups() for a in v)
        authors = ((Author(a[0]), int(a[1])) for a in authors)
        return AuthorList(authors)

    def author_affiliation(self, v):
        v = v.split('; ')
        affiliations = (re.search('^\((\d+)\) (.*)$', a).groups() for a in v)
        affiliations = {int(k): v for k, v in affiliations}
        return Affiliations(affiliations)

    def corresponding_author(self, v):
        return Author(v)

    def publication_year(self, v):
        return int(v)

    def abstract(self, v):
        raise IgnoreField()


class RecordProcessor(object):
    def process(self, record):
        if 'corresponding_author' in record:
            self.get_affiliation(record)

        # Convert back to strings
        record['authors'] = record['authors']
        record['author_affiliation'] = record['author_affiliation']
        record['corresponding_author'] = str(record['corresponding_author'])

        return record

    def get_affiliation(self, record):
        corresponding_author = record['corresponding_author']

        for author, affiliation in record['authors']:
            if corresponding_author == author:
                affiliation = record['author_affiliation'][affiliation]
                break
        else:
            raise ValueError('Could not find the corresponding author '
                             'affiliation in record {}'.format(record['id']))

        record['corresponding_author_affiliation'] = affiliation


def parse(fh):
    return Parser(ValueParser(), RecordProcessor()).parse(fh)



import itertools
import collections

import networkx as nx


def get_country(text):
    return text.rsplit(', ')[-1]


def get_countries(affiliations):
    return set(get_country(a) for a in affiliations)


def create(dataset):
    g = nx.Graph()

    affiliation_sets = dataset['author_affiliation']

    papers_count = collections.Counter()
    collaborations_count = collections.Counter()

    # Create graph
    for paper in affiliation_sets:
        countries = get_countries(paper.itervalues())
        g.add_nodes_from(countries)
        papers_count.update(countries)

        collaborations = list(itertools.combinations(countries, 2))
        collaborations_count.update(collaborations)
        g.add_edges_from(collaborations)

    # Set papers count
    for country, count in papers_count.iteritems():
        g.node[country]['papers'] = count

    # Set edge weight
    for (c1, c2), count in collaborations_count.iteritems():
        g[c1][c2]['weight'] = count

    return g



#!/usr/bin/env python

import os
from setuptools import setup, find_packages


class Setup(object):
    @staticmethod
    def read(fname, fail_silently=False):
        """
        Utility function to read the content of the given file.
        """
        try:
            return open(os.path.join(os.path.dirname(__file__), fname)).read()
        except:
            if not fail_silently:
                raise
            return ''

    @staticmethod
    def requirements(fname):
        """
        Utility function to create a list of requirements from the output of
        the pip freeze command saved in a text file.
        """
        packages = Setup.read(fname, fail_silently=True).split('\n')
        packages = (p.strip() for p in packages)
        packages = (p for p in packages if p and not p.startswith('#'))
        return list(packages)

    @staticmethod
    def get_files(*bases):
        """
        Utility function to list all files in a data directory.
        """
        for base in bases:
            basedir, _ = base.split('.', 1)
            base = os.path.join(os.path.dirname(__file__), *base.split('.'))

            rem = len(os.path.dirname(base)) + len(basedir) + 2

            for root, dirs, files in os.walk(base):
                for name in files:
                    yield os.path.join(basedir, root, name)[rem:]


setup(name='irco',
      version='0.1',
      description='International Research Collaboration Graphs',
      author='Jonathan Stoppani',
      author_email='jonathan@stoppani.name',
      url='https://github.com/GaretJax/irco',
      license='MIT',
      packages=find_packages(),
      package_dir={'irco': 'irco'},
      include_package_data=True,
      install_requires=Setup.requirements('requirements.txt'),
      entry_points=Setup.read('entry-points.ini', True))
